
let authHeader
let profile
let matchlist

async function getProfile () {
  const response = await fetch(
    "https://play.backgammongalaxy.com/user-profile/api/users/profiles/current",
    {
      headers: {
        Authorization: authHeader
      }
    }
  );
  return await response.json();
}

async function getMatchList (page) {
  const response = await fetch(
    `https://play.backgammongalaxy.com/stats/api/analyses/list/${profile.id}/${page}`,
    {
      headers: {
        Authorization: authHeader
      }
    }
  );
  return await response.json();
}


async function getMatchGames (matchId) {
  const response = await fetch(
    `https://play.backgammongalaxy.com/api/after_matches/${matchId}/games`,
    {
      headers: {
        Authorization: authHeader
      }
    }
  );
  return await response.json();
}

async function getMatch (matchId) {
  const response = await fetch(
    `https://play.backgammongalaxy.com/api/matches/${matchId}`,
    {
      headers: {
        Authorization: authHeader
      }
    }
  );
  return await response.json();
}

async function getGameData (matchId, gameIndex) {
  const response = await fetch(
    `https://play.backgammongalaxy.com/api/after_matches/${matchId}/games/${gameIndex}`,
    {
      headers: {
        Authorization: authHeader
      }
    }
  );
  return await response.json();
}

async function annotateMatch (matchListItem) {
  /*
  Take a simple matchListItem (just id, white, black) and return a fully annotated match object.

  this includes data from the getMatch API, and all game data.
  */

  // First annotate with info from getMatch API call
  const matchData = await getMatch(matchListItem.id)
  const matchInfo = matchData.data.attributes


  // Add in games with a poll to getMatchGames
  const matchGames = await getMatchGames(matchListItem.id)
  const match = await {
    ...matchListItem,
    length: matchInfo.length,
    clock: matchInfo.clock,
    whiteId: matchInfo.player1.id,
    blackId: matchInfo.player2.id,
    games: matchGames.data.items.map(game => {
      return {
        blackScore: game.home_previous_points,
        whiteScore: game.away_previous_points,
        game_index: game.game_index
      }
    })
  }
  match.games = await Promise.all(match.games.map(async game => {
    const events = await getGameData(match.id, game.game_index)
    game.events = events.data.events.map(e => {
      return {
        type: e.attributes.event_type,
        userId: e.attributes.user_id,
        data: e.attributes.data
      }
    })
    return game
  }))
  return match
}

function invertNumber (num) {
  /* 12/17 1/5 -> 13/8 24/20 */
  return 25 - num
}

function formatMovesArray (moves, isWhite) {
  // Take an array of moves, and iterate over in pairs. So an array of [12, 14, 18, 20] returns "12/14 18/20"
  return moves.reduce((acc, move, i, arr) => {
    if (i % 2 === 0) {
      let nextMove = arr[i + 1]
      if (isWhite) {
        move = invertNumber(move)
        nextMove = invertNumber(nextMove)
      }
      acc.push(`${move}/${nextMove}`)
    }
    return acc
  }, []).join(' ')
}

function formatDice (dice) {
  // Return a string of the dice, sorted in descending order
  return dice.sort((a, b) => b - a).join('')
}

function annotatedMatchToJelly (match) {
  /*
  Jelly format - 1 or 2 columns per line. Match looks like:

  ; [Site "BackgammonGalaxy"]
  ; [Match ID "21378425"]
  ; [Player 1 "nachtmare"]
  ; [Player 2 "commish223"]
  ; [Player 1 Elo "5,00/0"]
  ; [Player 2 Elo "4,00/0"]
  ; [Variation "Backgammon"]
  ; [Unrated "Off"]
  ; [Crawford "On"]
  ; [CubeLimit "1024"]

   5 point match

   Game 1
   nachtmare : 0                   commish223 : 0
    1) 61: 13/7 8/7                66: 13/7 13/7 13/7 13/7
    2) 65: 13/7 13/8               62: 13/7 24/22
    3) 62: 24/22 22/16             44: 7/3 7/3 6/2 6/2
    4) 44: 24/20 20/16 13/9 13/9   64: 24/20 20/14
    5)  Doubles => 2                Drops
        Wins 1 point

   */
  const lines = []

  // Do the semi-colon prefixed intro
  lines.push([`; [Site "BackgammonGalaxy"]`])
  lines.push([`; [Match ID "${match.id}"]`])
  lines.push([`; [Player 1 "${match.blackName}"]`])
  lines.push([`; [Player 2 "${match.whiteName}"]`])
  lines.push([`; [Player 1 Elo "${match.blackRating}"]`])
  lines.push([`; [Player 2 Elo "${match.whiteRating}"]`])
  lines.push([`; [Variation "Backgammon"]`])
  lines.push([`; [Unrated "Off"]`])
  lines.push([`; [Crawford "On"]`])
  lines.push([`; [CubeLimit "${match.games[0].events[0].data.cube_limit}"]`])
  lines.push([''])
  lines.push([`${match.length} point match`])
  lines.push([''])
  match.games.forEach(game => {
    lines.push([`Game ${game.game_index}`])
    lines.push([`${match.blackName} : ${game.blackScore}`, `${match.whiteName} : ${game.whiteScore}`])

    // black is always on the left of the pair, white on the right
    let actionPairs = ['', '']
    let lastDice
    let firstAction = true
    let actionIndex = undefined
    let gameLineCount = 0
    let cube = 1

    let pushLine = function () {
      gameLineCount += 1
      actionPairs[0] = `  ${gameLineCount}) ${actionPairs[0]}`
      lines.push(actionPairs)
      actionPairs = ['', '']
      firstAction = false
    }

    game.events.forEach(event => {

      actionIndex = event.userId === match.blackId ? 0 : 1
      if (event.type === 'game_started' || event.type === 'dice_rolled') {
        lastDice = formatDice(event.data.rolled_dice)
      } else if (event.type === 'move_commited' || event.type === 'move_committed') {  // yes, typo! we handle in case they fix it
        actionPairs[actionIndex] = `${lastDice}: ${formatMovesArray(event.data.moves, actionIndex === 1)}`
      } else if (event.type === 'double_requested') {
        cube *= 2
        actionPairs[actionIndex] = ` Doubles => ${cube}`
      } else if (event.type === 'double_accepted') {
        actionPairs[actionIndex] = ' Takes'
      } else if (event.type === 'double_rejected') {
        actionPairs[actionIndex] = ' Drops'
      } else if (event.type === 'turn_forfeited') {
        actionPairs[actionIndex] = `${lastDice}: `
      } else if (event.type === 'resigned') {
        actionPairs[actionIndex] = ' Resigns'
      } else if (event.type === 'game_over') {
        actionIndex = event.data.winner_user_id === match.blackId ? 0 : 1
        actionPairs[actionIndex] = ` Wins ${event.data.points_earned} point${event.data.points_earned === 1 ? '' : 's'}`
      }

      // if both actionPairs are not undefined, push them to the lines and empty actionPairs
      // also do this if first action is on white
      if ((actionPairs[0] && actionPairs[1]) || (firstAction && actionPairs[1])) {
        pushLine()
      }
    })

    // if this is the last game in the array, add "and the match" onto the last action
    if (game.game_index === match.games.length) {
      actionPairs[actionIndex] += ' and the match'
    }
    pushLine()
    lines.push([''])
  })

  return lines

}

function jellyToString (jelly) {
  // Get the longest first array item in the jelly list
  const longest = Math.max(...jelly.map(line => line[0].length)) + 3
  let output = ''
  jelly.forEach(line => {
    output += line.map(col => col.padEnd(longest)).join('') + '\n'
  })
  return output
}



chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  if (message.type === 'setAuthHeader') {
    if (authHeader !== message.authHeader) {
      authHeader = message.authHeader
      getProfile().then(data => {
        profile = data
        getMatchList(1).then(data => {
          matchlist = data.analyses.map(a => {
            return {
              id: a.matchId,
              whiteName: a.opponentName,
              blackName: profile.name,
              whiteRating: a.opponentRating,
              blackRating: a.userRating
            }
          })
        })
      })
    }
  }
});


window.addEventListener('load', function() {

  // Create a MutationObserver instance
  const observer = new MutationObserver((mutations) => {
    mutations.forEach((mutation) => {
      if (mutation.type === 'childList') {
        mutation.addedNodes.forEach((node) => {
          if (node.nodeType === Node.ELEMENT_NODE && node.tagName === 'DIV') {
            // If the added node is a div, find all images within it and attach the click event
            const images = node.querySelectorAll('img');
            images.forEach((img) => {
              if (img.src.includes('icon-download.svg')) {
                // get the img's parent's sibling
                const parent = img.parentElement
                parent.style.backgroundColor = 'rgb(21, 101, 192)'
                const a = parent.parentElement.querySelector('a')
                const matchId = a.href.split('/').pop()
                parent.addEventListener('click', async (e) => {
                  e.preventDefault()
                  e.stopPropagation()
                  const matchInfo = matchlist.find(match => match.id === Number(matchId))
                  const match = await annotateMatch(matchInfo)
                  const jelly = annotatedMatchToJelly(match)
                  const oneFile = jellyToString(jelly)
                  const blob = new Blob([oneFile], { type: 'text/plain' });
                  const url = URL.createObjectURL(blob);
                  const link = document.createElement('a');
                  link.href = url;
                  link.download = `${match.id}.txt`;
                  document.body.appendChild(link);
                  link.click();
                  document.body.removeChild(link);
                  URL.revokeObjectURL(url);
                })
              }
            });
          }
        });
      }
    });
  })

// Start observing the document with the configured parameters
  observer.observe(document.body, {
    childList: true, // Observe direct children of the target node
    subtree: true, // Also observe all descendants of the target node
  });
});
